
package book.jmx.examples;

import java.util.*;
import javax.management.*;

public class Queries {

  public static void main(String[] args) throws Exception {
    MBeanServer server = MBeanServerFactory.createMBeanServer();
    
    ObjectName john = new ObjectName("DomainA:name=john");
    ObjectName mike = new ObjectName("DomainA:name=mike");
    ObjectName xena = new ObjectName("DomainA:name=xena");
    
    server.createMBean("book.jmx.examples.User", john);
    server.createMBean("book.jmx.examples.User", mike);
    server.createMBean("book.jmx.examples.User", xena);
    
    server.setAttribute(john, new Attribute("Name", "John"));
    server.setAttribute(mike, new Attribute("Name", "Mike"));
    server.setAttribute(xena, new Attribute("Name", "Xena"));
    
    server.setAttribute(john, 
        new Attribute("Address", "King's Road 11"));
    server.setAttribute(mike, 
        new Attribute("Address", "Strawberry St. 5"));
    server.setAttribute(xena, 
        new Attribute("Address", "Strawberry St. 12"));

    testQueries(server);
  }

  static void testQueries(MBeanServer server) throws Exception {
      
    QueryExp exp = 
        Query.match(Query.attr("Name"), Query.value("John"));
    
    ObjectName scope = new ObjectName("DomainA:*");    
    Set set      = server.queryNames(scope, exp);    
    Iterator it  = set.iterator();
    
    System.out.println("MBeans with attribute Name = 'John'\n");
    
    while (it.hasNext()) {
      System.out.println(it.next());    
    }

    scope = new ObjectName("DomainA:*");    
    exp   = Query.initialSubString(Query.attr("Address"),
                                   Query.value("Strawberry"));
        
    set   = server.queryNames(scope, exp);    
    it    = set.iterator();
    
    System.out.println(
        "\nMBeans with Address startsWith 'Strawberry'\n");
    
    while (it.hasNext()) {
      System.out.println(it.next());
    }
  }
  
}

